# -*- coding: utf8 -*
#
#	Gedit plugin to open the current file in a different application.
#
#	Copyright (C) 2006, Markus Jonsson <marjo691@student.liu.se>
#
#	Based on "Open Terminal Here" by Ramzi Ferchichi
#	Copyright (C) 2005, Ramzi Ferchichi <ramzi@gafouri.net>
#
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA	02110-1301	USA
#

import gnomevfs

import gedit
import gtk
import gconf

import os
import os.path

open_with_ui_start = """
<ui>
	<popup name="NotebookPopup" action="NotebookPopupAction">
		<placeholder name="NotebookPupupOps_1">
				"""
				
open_with_ui_end = """
		</placeholder>
	</popup>
</ui>
"""


#
# TODO:
#	- Submenu when items >= 3
#	- Handle non file:// uri's
#	- Thorough error checking
#	- I18N
#

class OpenWithPlugin(gedit.Plugin):

	def __init__(self):
		gedit.Plugin.__init__(self)
		# self.conf_client = gconf.client_get_default()
	
	def launcher_cb(self, action, window, app):
		active_doc = window.get_active_document()
		uri = active_doc.get_uri().replace("file://","")
		os.system(app + " " + uri + " &")
	
	def get_apps_from_mime(self, mime):
		apps = gnomevfs.mime_get_all_applications(mime)
		ret = []
		for app in apps:
			if 'gedit' not in app:
				ret+=app[0:3],
		
		return ret
	
	def active_tab_changed_cb(self, window, tab):
		
		mime = window.get_active_document().get_mime_type()
		apps = self.get_apps_from_mime(mime)
		
		# retrive current data
		data = window.get_data("OpenWithPluginInfo")
		
		# remove current menu
		ui_manager = window.get_ui_manager()
		action_group = data["action_group"]
		
		if not data["ui_id"] is None:
			ui_manager.remove_ui(data["ui_id"])
		
		ui_str = open_with_ui_start;
		
		actions = []
		for app in apps:
			action = gtk.Action(app[1].replace(' ',''),
													"Open With \"" + app[1] + "\"",
													"Open With \"" + app[1] + "\"",
													gtk.STOCK_OPEN)
			action.connect("activate", self.launcher_cb, window, app[2])
			
			print app
			
			action_group.add_action(action)
			ui_str += "<menuitem action=\""+app[1].replace(' ','')+"\" />";
		
		ui_str += open_with_ui_end;
		
		data["ui_id"] = ui_manager.add_ui_from_string(ui_str)
		
		# set current data
		window.set_data("OpenWithPluginInfo",data)
		
	
	def activate(self, window):	
		
		action_group = gtk.ActionGroup("OpenWithPluginActions")

		ui_manager = window.get_ui_manager()
		ui_manager.insert_action_group(action_group, 0)
		
		windowdata = dict()
		windowdata["action_group"] = action_group
		windowdata["ui_id"] = None
		window.set_data("OpenWithPluginInfo", windowdata)
		
		chcb = window.connect("active-tab-changed", self.active_tab_changed_cb)
		window.set_data("OpenWithPluginHandler", chcb)

	def deactivate(self, window):
		windowdata = window.get_data("OpenWithPluginInfo")
		
		ui_manager = window.get_ui_manager()
		ui_manager.remove_ui(windowdata["ui_id"])
		ui_manager.remove_action_group(windowdata["action_group"])
		ui_manager.ensure_update()
		
		chcb = window.get_data("OpenWithPluginHandler")
		window.disconnect(chcb)
		window.set_data("OpenWithPluginHandler", None)
		

	def is_valid_doc(self, doc):
		return bool(doc and
								doc.get_uri() and
								doc.get_uri().startswith("file://"))
		

	def update_ui(self, window):
		windowdata = window.get_data("OpenWithPluginInfo")
		active_doc = window.get_active_document()
		windowdata["action_group"].set_sensitive(self.is_valid_doc(active_doc))
		pass
		
